-- Most of the comments to the code (and the "well written" part of it :-) are by Phanx, whom I thanks for the helps and patience in explaining. 
-- The layout of the input frame was inspired by the one of : http://www.wowinterface.com/downloads/info21884-AutoRun_Reminder.html

AutopsyCFG = AutopsyCFG or {}

local cmd, out -- cmd args
local prgname = "|cffffd200autopsy|r"
local string_format = string.format
local timeStamp, event, sourceGUID, sourceName, destGUID, destName, prefixParam1, prefixParam2, suffixParam1, suffixParam2
local autopsy_msg, autopsy_msg_solo, autopsy_chn -- output vars

local function color(destName)
	if not UnitExists(destName) then return string_format("\124cffff0000%s\124r", destName) end
	local _, class = UnitClass(destName)
	local color = _G["RAID_CLASS_COLORS"][class]
	return string_format("\124cff%02x%02x%02x%s\124r", color.r*255, color.g*255, color.b*255, destName)
end

local function linkname(destName)

return string_format("\124Hplayer:%s:1:WHISPER:%s\124h[%s]\124h", destName, destName, color(destName) )

end

local function Autopsy_OUTPUT(event, sourceName, destName, prefixParam1, prefixParam2, suffixParam1, suffixParam2)

	if (event == "SWING_DAMAGE") then		
		autopsy_msg = string_format("autopsy: [%s] killed [%s] with %s Melee overkill %s", sourceName, destName, prefixParam1, prefixParam2)	
		autopsy_msg_solo = string_format("\124cffffd200autopsy\124r: %s killed %s with %s Melee overkill %s", linkname(sourceName), linkname(destName), prefixParam1, prefixParam2)
	else 
		autopsy_msg = string_format("autopsy: [%s] killed [%s] with %s damage of %s overkill %s", sourceName, destName, suffixParam1, GetSpellLink(prefixParam1), suffixParam2)
		autopsy_msg_solo = string_format("\124cffffd200autopsy\124r: %s killed %s with %s damage of %s overkill %s", linkname(sourceName), linkname(destName), suffixParam1, GetSpellLink(prefixParam1), suffixParam2)
	end
	
	-- find the right GROUP, thanks to ideas and code in the posts of http://www.curse.com/addons/wow/fatality
	if AutopsyCFG["CHAT"] == "GROUP" then 
	
		-- solo
		if (GetNumGroupMembers(LE_PARTY_CATEGORY_HOME) == 0 and GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) == 0) then
			autopsy_chn = "SELF"
		
		-- party or raid
		elseif (GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) == 0 and GetNumGroupMembers(LE_PARTY_CATEGORY_HOME) ~= 0) then
			if  IsInRaid() then 
				autopsy_chn = "RAID"
			else
				autopsy_chn = "PARTY"
			end
		
		-- instance LFG/LFR
		elseif (GetNumGroupMembers(LE_PARTY_CATEGORY_HOME) == 0 and GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) ~= 0) then
			autopsy_chn = "INSTANCE_CHAT"
		end			
	
	else 	
		-- not group chat, i.e. GUILD, OFFICER
		autopsy_chn = AutopsyCFG["CHAT"]
	
	end	
			
	if 	autopsy_chn == "SELF" then 
		print(autopsy_msg_solo)	

	else 
		SendChatMessage(autopsy_msg, autopsy_chn)	

	end 		
end

-- Frame to register the global vars at the ADDON_LOADED event.	
local frame_al = CreateFrame("FRAME", "autopsyframe_al")
frame_al:RegisterEvent("ADDON_LOADED")	
frame_al:SetScript("OnEvent", function(self, event, arg1, ...)
	if event == "ADDON_LOADED" and arg1 == "Autopsy" then	
		
		AutopsyCFG["enabled"] = AutopsyCFG["enabled"] or true
		AutopsyCFG["CHAT"] = AutopsyCFG["CHAT"] or "SELF"

		if AutopsyCFG["enabled"] == true then 
			print(string_format("%s: enabled on [|cffff0000%s|r] channel", prgname, AutopsyCFG["CHAT"]))
		else 
			print(string_format("%s: disabled. Type /%s enable to start", prgname, prgname))
		end
		
		self:UnregisterEvent("ADDON_LOADED")		
	end	
end)

-- Frame to parse CLEU	
local frame_cleu = CreateFrame("FRAME", "autopsyframe_cleu")
frame_cleu:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu:RegisterEvent("PLAYER_ENTERING_WORLD")
frame_cleu:SetScript("OnEvent", function(self, event, ...)
	
	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then	
		
		if AutopsyCFG["enabled"] == false then
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		return
	end

	timeStamp, event, _, sourceGUID, sourceName, _, _, destGUID, destName, _, _, prefixParam1, prefixParam2, _, suffixParam1, suffixParam2 = ...	
	
    -- proceed with CLEU handling
	if not playerGUID then playerGUID = UnitGUID("player") end
	
	if destGUID == playerGUID or (destName and UnitClass(destName)) then

		-- prevent the case the sourceName is nil (sometimes it is happened)
		sourceName = sourceName or "Someone"
		
		if event == "SWING_DAMAGE" then
			if prefixParam2 > 0 then Autopsy_OUTPUT(event, sourceName, destName, prefixParam1, prefixParam2, suffixParam1, suffixParam2) end
		elseif event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "RANGE_DAMAGE" then
			if suffixParam2 > 0 then Autopsy_OUTPUT(event, sourceName, destName, prefixParam1, prefixParam2, suffixParam1, suffixParam2) end
		end
	end
end)
	
SLASH_AUTOPSY1 = "/autopsy";
SlashCmdList["AUTOPSY"] = function(args) 
	cmd, out = string.split(" ", args, 2)
	
	if cmd == "enable" then
		AutopsyCFG["enabled"] = true
		print(string_format("%s: log monitor is enabled",prgname))
		frame_cleu:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	elseif cmd == "disable" then
		AutopsyCFG["enabled"] = false
		print(string_format("%s: log monitor is disabled",prgname))
		frame_cleu:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	elseif cmd == "out" then
		if out then out = out:upper() end			
		if out == "OFFICER" or out == "GUILD" or out == "GROUP" or out == "SELF" then 
			AutopsyCFG["CHAT"] = out 
			print(string_format("%s: OUT channel is now |cffff0000%s|r",prgname, out))
		end	
		
	elseif cmd == "testmelee" then
		event = "SWING_DAMAGE"
		sourceName = "Brown Marmot"
		destName = UnitName("player") 
		prefixParam1 = math.random(50000,100000)
		prefixParam2 = math.random(50000)
		Autopsy_OUTPUT(event, sourceName, destName, prefixParam1, prefixParam2, nil, nil)
	
	elseif cmd == "testspell" then 		
		event = "SPELL_DAMAGE"
		sourceName = "Brown Marmot"
		destName = UnitName("player") 
		suffixParam1 = math.random(50000,100000)
		suffixParam2 = math.random(50000)
		prefixParam1 = "30455"  -- ice lance
		Autopsy_OUTPUT(event, sourceName, destName, prefixParam1, nil, suffixParam1, suffixParam2)

	else
		if AutopsyCFG["enabled"] == false then			
			print(string_format("%s: |cffff0000disabled|r. Type /%s enable to ", prgname, prgname)) 
		else
			print(string_format("%s: |cff228B22enabled|r on [|cffff0000%s|r] channel ", prgname, AutopsyCFG["CHAT"])) 
		end 
		
		print(prgname .. " commands")
		print("/autopsy - This help")
		print("/autopsy out [self|group|guild|officer] - Channel") 
		print("/autopsy enable - Enable the log monitor")
		print("/autopsy disable - Disable the log monitor")
		
		return
	end
end